//ELLIOT GREENFIELD 2017

#include "ofApp.h"
#include "ofxMaxim.h"
#include "maximilian.h"



Params param;


void Params::setup(){
    
    eCenter = ofPoint( ofGetWidth() /2, ofGetHeight() /2);
    eRad = 800;                 // the params for the particle can be changed to give different
    
    velRad = 0;
    lifeTime = 0.50;                 // lifetime of the particle
    
    rotate = 500;
    force = 10000;
    spinning = 1000;
    friction = 0.5;           // THESE CAN BE CHANGED TO GIVE DIFFERENT EFFECT
}


Particle::Particle(){
    
    live = false;
    
}

ofPoint randomPointInCircle( float maxRad){
    
    ofPoint pnt;
    float rad = ofRandom( 0, maxRad);
    float angle = ofRandom( 0, M_TWO_PI);
    pnt.x = cos (angle) * rad;
    pnt.y = sin (angle) * rad;
    return pnt;
    
}


void Particle::setup() {
    
    
    fftSmooth = new float [8192];       // I FOUND DOING THE FFT INSIDE THE PARTICLE SETUP
    for (int i = 0; i < 8192; i++){     // WORKED ALOT EASIER THAN DOING IT INSIDE THE OFDRAW
        fftSmooth[i] = 0;
    }
    
    
    bands = 64;   // fft bands
    
    
    
    pos = param.eCenter + randomPointInCircle(param.eRad);
    vel = randomPointInCircle(param.velRad);
    time = 0;
    lifeTime = param.lifeTime;
    live = true;
    
}

void Particle::update(float dt){
    
    float * value = ofSoundGetSpectrum(bands);
    for (int i = 0; i < bands; i++){
        fftSmooth[i] *= 0.9f;
        if(fftSmooth[i] < value[i]){
            fftSmooth[i] = value[i];
        }
    }
    
    
    
    
    if (live){
        vel.rotate(0,0, param.rotate * dt);
        
        ofPoint acc;
        ofPoint delta = pos - param.eCenter;
        float len = delta.length();
        if (ofInRange(len, 0, param.eRad)){   // changes the force of which the particles are
            delta.normalize();                  //moving 
            
            acc += delta * param.force;
            
            acc.x += -delta.y * param.spinning;
            acc.y += delta.x * param.spinning;
            
            
        }
        
        vel += acc * dt;
        vel *= (1 - param.friction);
        pos += vel * dt;
        time += dt;
        
        if (time>= lifeTime){
            live = false;
        }
    }
}


void Particle::draw(){
    if (live){
        
        
        
        
        
        
        
        float size = ofMap(fabs(time- lifeTime/2), 0, lifeTime/2, 3, 1);
        
        ofColor color = ofColor::red;
        float hue = ofMap (time,0,lifeTime,128,255); // SETS THE COLOUR THAT IS CHANGED
        color.setHue(hue);
        ofSetColor(color);
    
        
        for (int i = 0; i < bands; i++){
            ofDrawCircle(pos,-(fftSmooth[i]/2)); // draws the particle to be effected by the fft
        }
  
    }
}


//--------------------------------------------------------------
void ofApp::setup(){
    
    song2.load("/Users/user/Desktop/hammer.mp3"); // loads song into program
    song2.setVolume(0.5);
    song2.play();                       //plays loaded song
    song2.setLoop(true);            // sets the loop so that it plays whenever it finishes

    fftSmooth = new float [8192];     // creates a new fft
    for (int i = 0; i < 8192; i++){
        fftSmooth[i] = 0;
    }
                    
    
    bands = 64;    // sets bands of fft
    
    
    
    
    ofSetFrameRate(60);  // sets framerate of programme
    
    int w = ofGetWidth();
    int h = ofGetHeight();
    fbo.allocate ( w , h, GL_RGB32F_ARB);
    
    fbo.begin();
    ofBackground(0, 0, 0); // sets background to black 
    fbo.end();
    
    param.setup();
    history = 0.9;
    
    time0 = ofGetElapsedTimef();
    
    bornRate = 2500;
    bornCount = 0;

}

//--------------------------------------------------------------
void ofApp::update(){

    
 
    
    
    
    
    
    float time = ofGetElapsedTimef();
    float dt = ofClamp( time - time0, 0 , 0.1);  // updating the screen with each particle
    time0 = time;
    
    
    int i =0;
    while (i < p.size()){
        if( !p[i].live){
            p.erase(p.begin() + i);
            
        }
        
        else{ i++;
        }
    }
  
    bornCount += dt * bornRate;
    if (bornCount >= 1){
        
        int bornN = int( bornCount);
        bornCount -= bornN;
        for (int i =0; i<bornN; i++){
            Particle newP;
            newP.setup();
            p.push_back(newP);
            
        }
        
    }
    
    for (int i=0; i<p.size(); i++){
        p[i].update(dt);
    }
    
    
    
}







//--------------------------------------------------------------
void ofApp::draw(){
    
    
    
    
    ofBackground(0,0,0);
    
    fbo.begin();
    
    ofEnableAlphaBlending();
    
    float alpha = (1-history) * 255;         // This changes the colour of the particles
    ofSetColor(0, 0, 0,alpha);
    ofFill();
    ofRect ( 0, 0 , ofGetWidth(), ofGetHeight() );
    
    ofDisableAlphaBlending();
    
    ofFill();
    for(int i=0; i<p.size(); i++){
        p[i].draw();
        
    }
    
    fbo.end();
    
    ofSetColor ( 255,255,255);
    fbo.draw(0,0);

}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

  
    
    
}
